#include <stdlib.h>

#include <pspkernel.h>
#include <pspdebug.h>
#include <math.h>
#include <time.h>
#include <libge/libge.h>

#include "zombis.h"

PSP_MODULE_INFO("LibGE Sample", PSP_MODULE_USER, 0, 1);
PSP_HEAP_SIZE_KB(2048);

float deg2rad(float deg) {return deg/180*M_PI;}
float rad2deg(float rad) {return rad*180/M_PI;}

float getAngle(float x, float y, float x0, float y0)
{
	float prem = geAbs(x - x0);
	float deux = geAbs(y - y0);
	
	float angle = rad2deg(atan(deux / prem));
	
	return angle;
}


typedef struct {
	float vie;
	int rechargement, armeChargee, tirs, tirsDispo;
	float angleBras;
	bool recharge;
	ge_Image *img;
} Joueur;

Joueur creerJoueur(ge_Image *img, int tirsDispo, float angle)
{
	Joueur temp;
	temp.img = img;
	
	temp.vie = 100;
	temp.rechargement = 0;
	temp.armeChargee = 1;
	temp.tirs = 0;
	temp.tirsDispo = tirsDispo;
	
	temp.angleBras = angle;
	
	temp.recharge = false;
	
	return temp;
}

typedef struct {
	float x, y;
	float avX, avY;
	float angle;
	int isVisible;
	ge_Image *img;
} Balle;

Balle creerTir(float angle)
{
	Balle tir;
	
	tir.img = geLoadImage("tir.png");
	tir.x = 80 + rand() % 4;
	tir.y = 108 + rand() % 6;
	
	tir.avX = cos(deg2rad(angle)) * 8;
	tir.avY = sin(deg2rad(angle)) * 8;
	
	tir.angle = angle;
	
	tir.isVisible = true;
	
	return tir;
}

/*bool isOutOfScreen(float x, float y, float l, float h)
{
	if (x - l < 0 || x > 480 || y - h < 0 || y > 272) return true;
	
	return false;
}*/

void DoLog(const char *msg)
{
	FILE *file = fopen("log.txt", "w+");
	
	fwrite(msg,1, sizeof(msg), file);
	
	fclose(file);
}

int main(int args, char** argc)
{
	pspDebugScreenInit();
	geInit();
	
	srand(time(NULL));
	
	geClearColor(RGB(0,255,255));
	
	ge_Keys* Touches = geCreateKeys();

	Joueur heros;
	heros = creerJoueur(geLoadImage("img.png") , 7, 0);
	
	int i;
	int j;
	
	Balle tirs[heros.tirsDispo];
	for (i = 0; i < heros.tirsDispo; i++)
	  tirs[i].isVisible = false;
	
	
	ge_Image *zombiImg = geCreateSurface(25, 40, RGB(255, 0, 0));
	Zombi ennemis[30];
	for (i = 0; i < 30; i++)
		ennemis[i] = creerZombie(1, 500 + 300 * i, 227, zombiImg);
	
	SceCtrlData pad;

	while (1)
	{
		geReadKeys(Touches);
		geClearScreen();
		sceCtrlReadBufferPositive(&pad, 1);// Bloque le FPS a 60
		
		if ((Touches->pressed[GEK_UP] || pad.Ly < 96) && heros.angleBras < 75)
		{
			heros.angleBras++;
		}
		else if ((Touches->pressed[GEK_DOWN] || pad.Ly > 160) && heros.angleBras > -75)
		{
			heros.angleBras--;
		}
		
		if (Touches->pressed[GEK_SQUARE] && heros.recharge == false) heros.recharge = true;
		
		
		if (Touches->pressed[GEK_CROSS] && heros.armeChargee == 1 && heros.recharge == false)
		{
			tirs[heros.tirs] = creerTir(heros.angleBras);
			heros.tirs++;
			
			heros.armeChargee = 0;
		}
		
		if (Touches->released[GEK_CROSS] && heros.armeChargee == 0) heros.armeChargee = 1;
		
		if (heros.tirs >= heros.tirsDispo)
		{
			heros.recharge = true;
		}
		
		if (heros.recharge == true)
		{
			heros.rechargement++;
			pspDebugScreenSetXY(3, 5);
			pspDebugScreenPrintf("Reloading...");
			if (heros.rechargement >= 100)
			{
				heros.rechargement = 0;
				heros.tirs = 0;
				heros.recharge = false;
			}	
		}
		
		for (i = 0; i < heros.tirs; i++)
		{
			if (tirs[i].isVisible == true)
			{
				if (isOutOfScreen(tirs[i].x, tirs[i].y, tirs[i].img->width, tirs[i].img->height) == true) tirs[i].isVisible = false;
				
			tirs[i].x += tirs[i].avX;
			tirs[i].y -= tirs[i].avY;
			
				geDrawImageRotated(tirs[i].x, tirs[i].y, tirs[i].img, tirs[i].angle, false);
			}
		}
		
		for (i = 0; i < 30; i++)
		{
		if (isOutOfScreen(ennemis[i].x, ennemis[i].y, ennemis[i].L, ennemis[i].H) == false)
			ennemis[i].visible = true;
		
		ennemis[i].x -= 0.5;
		//if (ennemis[i].vie <= 0) ennemis[i].visible = false; // On retire les ennemis morts
			
			if (ennemis[i].visible == true)
			{
					for (j = 0; j < heros.tirs; j++)
					{
						if (tirs[j].isVisible == true && doesCollisionHappend(ennemis[i].x,ennemis[i].y,ennemis[i].L,ennemis[i].H,tirs[j].x,tirs[j].y,tirs[j].img->width,tirs[j].img->height))
						{
							ennemis[i].vie -= 20;
							
							pspDebugScreenSetXY(3, 7);
							pspDebugScreenPrintf("Coll!!!!!");
						}
					}
					geDrawImage(ennemis[i].x,ennemis[i].y,ennemis[i].img);
					geFillRectScreen(ennemis[i].x,ennemis[i].y-20, 32, 6, RGB(0,255,0));
					geFillRectScreen(ennemis[i].x+ 1,ennemis[i].y-19,ennemis[i].vie*30/100 , 4, RGB(255,0,0));
			}
			
		}
		
		geDrawImageRotated(40, 100, heros.img, heros.angleBras, false);
		
		pspDebugScreenSetXY(20, 0);
		pspDebugScreenPrintf("AngleBras %f", heros.angleBras);
		
		geFpsSetLimit(30);
		sceDisplayWaitVblankStart();//Ralentit le programme
		geSwapBuffers();
		pspDebugScreenSetXY(0, 0);
		pspDebugScreenPrintf("FPS: %d", geFps());
	}
}
